# Generated code. Do not modify.
# flake8: noqa: F405

from .array_value import ArrayValue
from .array_variable import ArrayVariable
from .boolean_value import BooleanValue
from .boolean_variable import BooleanVariable
from .color_value import ColorValue
from .color_variable import ColorVariable
from .content_text import ContentText
from .content_url import ContentUrl
from .dict_value import DictValue
from .dict_variable import DictVariable
from .div import Div
from .div_absolute_edge_insets import DivAbsoluteEdgeInsets
from .div_accessibility import (
    DivAccessibility,
    DivAccessibilityMode,
    DivAccessibilityType,
)
from .div_action import DivAction, DivActionMenuItem, DivActionTarget
from .div_action_animator_start import DivActionAnimatorStart
from .div_action_animator_stop import DivActionAnimatorStop
from .div_action_array_insert_value import DivActionArrayInsertValue
from .div_action_array_remove_value import DivActionArrayRemoveValue
from .div_action_array_set_value import DivActionArraySetValue
from .div_action_clear_focus import DivActionClearFocus
from .div_action_copy_to_clipboard import DivActionCopyToClipboard
from .div_action_copy_to_clipboard_content import DivActionCopyToClipboardContent
from .div_action_custom import DivActionCustom
from .div_action_dict_set_value import DivActionDictSetValue
from .div_action_download import DivActionDownload
from .div_action_focus_element import DivActionFocusElement
from .div_action_hide_tooltip import DivActionHideTooltip
from .div_action_scroll_by import DivActionScrollBy, DivActionScrollByOverflow
from .div_action_scroll_destination import DivActionScrollDestination
from .div_action_scroll_to import DivActionScrollTo
from .div_action_set_state import DivActionSetState
from .div_action_set_stored_value import DivActionSetStoredValue
from .div_action_set_variable import DivActionSetVariable
from .div_action_show_tooltip import DivActionShowTooltip
from .div_action_submit import (
    DivActionSubmit,
    DivActionSubmitRequest,
    RequestHeader,
    RequestMethod,
)
from .div_action_timer import DivActionTimer, DivActionTimerAction
from .div_action_typed import DivActionTyped
from .div_action_update_structure import DivActionUpdateStructure
from .div_action_video import DivActionVideo, DivActionVideoAction
from .div_alignment_horizontal import DivAlignmentHorizontal
from .div_alignment_vertical import DivAlignmentVertical
from .div_animation import DivAnimation, DivAnimationName
from .div_animation_direction import DivAnimationDirection
from .div_animation_interpolator import DivAnimationInterpolator
from .div_animator import DivAnimator
from .div_animator_base import DivAnimatorBase
from .div_appearance_set_transition import DivAppearanceSetTransition
from .div_appearance_transition import DivAppearanceTransition
from .div_aspect import DivAspect
from .div_background import DivBackground
from .div_base import DivBase
from .div_blend_mode import DivBlendMode
from .div_blur import DivBlur
from .div_border import DivBorder
from .div_change_bounds_transition import DivChangeBoundsTransition
from .div_change_set_transition import DivChangeSetTransition
from .div_change_transition import DivChangeTransition
from .div_circle_shape import DivCircleShape
from .div_cloud_background import DivCloudBackground
from .div_collection_item_builder import (
    DivCollectionItemBuilder,
    DivCollectionItemBuilderPrototype,
)
from .div_color_animator import DivColorAnimator
from .div_container import (
    DivContainer,
    DivContainerLayoutMode,
    DivContainerOrientation,
    DivContainerSeparator,
)
from .div_content_alignment_horizontal import DivContentAlignmentHorizontal
from .div_content_alignment_vertical import DivContentAlignmentVertical
from .div_corners_radius import DivCornersRadius
from .div_count import DivCount
from .div_currency_input_mask import DivCurrencyInputMask
from .div_custom import DivCustom
from .div_data import DivData, DivDataState
from .div_default_indicator_item_placement import DivDefaultIndicatorItemPlacement
from .div_dimension import DivDimension
from .div_disappear_action import DivDisappearAction
from .div_download_callbacks import DivDownloadCallbacks
from .div_drawable import DivDrawable
from .div_edge_insets import DivEdgeInsets
from .div_evaluable_type import DivEvaluableType
from .div_extension import DivExtension
from .div_fade_transition import DivFadeTransition
from .div_filter import DivFilter
from .div_filter_rtl_mirror import DivFilterRtlMirror
from .div_fixed_count import DivFixedCount
from .div_fixed_length_input_mask import (
    DivFixedLengthInputMask,
    DivFixedLengthInputMaskPatternElement,
)
from .div_fixed_size import DivFixedSize
from .div_fixed_translation import DivFixedTranslation
from .div_focus import DivFocus, DivFocusNextFocusIds
from .div_font_weight import DivFontWeight
from .div_function import DivFunction
from .div_function_argument import DivFunctionArgument
from .div_gallery import (
    DivGallery,
    DivGalleryCrossContentAlignment,
    DivGalleryOrientation,
    DivGalleryScrollbar,
    DivGalleryScrollMode,
)
from .div_gif_image import DivGifImage
from .div_grid import DivGrid
from .div_image import DivImage
from .div_image_background import DivImageBackground
from .div_image_scale import DivImageScale
from .div_indicator import DivIndicator, DivIndicatorAnimation
from .div_indicator_item_placement import DivIndicatorItemPlacement
from .div_infinity_count import DivInfinityCount
from .div_input import (
    DivInput,
    DivInputAutocapitalization,
    DivInputEnterKeyType,
    DivInputKeyboardType,
    DivInputNativeInterface,
)
from .div_input_filter import DivInputFilter
from .div_input_filter_expression import DivInputFilterExpression
from .div_input_filter_regex import DivInputFilterRegex
from .div_input_mask import DivInputMask
from .div_input_mask_base import DivInputMaskBase
from .div_input_validator import DivInputValidator
from .div_input_validator_base import DivInputValidatorBase
from .div_input_validator_expression import DivInputValidatorExpression
from .div_input_validator_regex import DivInputValidatorRegex
from .div_layout_provider import DivLayoutProvider
from .div_line_style import DivLineStyle
from .div_linear_gradient import DivLinearGradient, DivLinearGradientColorPoint
from .div_match_parent_size import DivMatchParentSize
from .div_neighbour_page_size import DivNeighbourPageSize
from .div_nine_patch_background import DivNinePatchBackground
from .div_number_animator import DivNumberAnimator
from .div_page_content_size import DivPageContentSize
from .div_page_size import DivPageSize
from .div_page_transformation import DivPageTransformation
from .div_page_transformation_overlap import DivPageTransformationOverlap
from .div_page_transformation_slide import DivPageTransformationSlide
from .div_pager import DivPager, DivPagerItemAlignment, DivPagerOrientation
from .div_pager_layout_mode import DivPagerLayoutMode
from .div_patch import DivPatch, DivPatchChange, DivPatchMode
from .div_percentage_size import DivPercentageSize
from .div_percentage_translation import DivPercentageTranslation
from .div_phone_input_mask import DivPhoneInputMask
from .div_pivot import DivPivot
from .div_pivot_fixed import DivPivotFixed
from .div_pivot_percentage import DivPivotPercentage
from .div_point import DivPoint
from .div_radial_gradient import DivRadialGradient, DivRadialGradientColorPoint
from .div_radial_gradient_center import DivRadialGradientCenter
from .div_radial_gradient_fixed_center import DivRadialGradientFixedCenter
from .div_radial_gradient_radius import DivRadialGradientRadius
from .div_radial_gradient_relative_center import DivRadialGradientRelativeCenter
from .div_radial_gradient_relative_radius import (
    DivRadialGradientRelativeRadius,
    DivRadialGradientRelativeRadiusValue,
)
from .div_rotation_transformation import DivRotationTransformation
from .div_rounded_rectangle_shape import DivRoundedRectangleShape
from .div_scale_transition import DivScaleTransition
from .div_select import DivSelect, DivSelectOption
from .div_separator import (
    DelimiterStyleOrientation,
    DivSeparator,
    DivSeparatorDelimiterStyle,
)
from .div_shadow import DivShadow
from .div_shape import DivShape
from .div_shape_drawable import DivShapeDrawable
from .div_sight_action import DivSightAction
from .div_size import DivSize
from .div_size_unit import DivSizeUnit
from .div_size_unit_value import DivSizeUnitValue
from .div_slide_transition import DivSlideTransition, DivSlideTransitionEdge
from .div_slider import DivSlider, DivSliderRange, DivSliderTextStyle
from .div_solid_background import DivSolidBackground
from .div_state import DivState, DivStateState
from .div_stretch_indicator_item_placement import DivStretchIndicatorItemPlacement
from .div_stroke import DivStroke
from .div_stroke_style import DivStrokeStyle
from .div_stroke_style_dashed import DivStrokeStyleDashed
from .div_stroke_style_solid import DivStrokeStyleSolid
from .div_switch import DivSwitch
from .div_tabs import (
    DivTabs,
    DivTabsItem,
    DivTabsTabTitleDelimiter,
    DivTabsTabTitleStyle,
    TabTitleStyleAnimationType,
)
from .div_text import (
    AccessibilityType,
    DivText,
    DivTextEllipsis,
    DivTextImage,
    DivTextRange,
    DivTextTruncate,
    ImageAccessibility,
    ImageIndexingDirection,
)
from .div_text_alignment_vertical import DivTextAlignmentVertical
from .div_text_gradient import DivTextGradient
from .div_text_range_background import DivTextRangeBackground
from .div_text_range_border import DivTextRangeBorder
from .div_text_range_mask import DivTextRangeMask
from .div_text_range_mask_base import DivTextRangeMaskBase
from .div_text_range_mask_particles import DivTextRangeMaskParticles
from .div_text_range_mask_solid import DivTextRangeMaskSolid
from .div_timer import DivTimer
from .div_tooltip import DivTooltip, DivTooltipPosition
from .div_tooltip_mode import DivTooltipMode
from .div_tooltip_mode_modal import DivTooltipModeModal
from .div_tooltip_mode_non_modal import DivTooltipModeNonModal
from .div_transform import DivTransform
from .div_transformation import DivTransformation
from .div_transition_base import DivTransitionBase
from .div_transition_selector import DivTransitionSelector
from .div_transition_trigger import DivTransitionTrigger
from .div_translation import DivTranslation
from .div_translation_transformation import DivTranslationTransformation
from .div_trigger import DivTrigger, DivTriggerMode
from .div_typed_value import DivTypedValue
from .div_variable import DivVariable
from .div_video import DivVideo
from .div_video_scale import DivVideoScale
from .div_video_source import DivVideoSource, DivVideoSourceResolution
from .div_visibility import DivVisibility
from .div_visibility_action import DivVisibilityAction
from .div_wrap_content_size import DivWrapContentSize
from .end_destination import EndDestination
from .index_destination import IndexDestination
from .integer_value import IntegerValue
from .integer_variable import IntegerVariable
from .number_value import NumberValue
from .number_variable import NumberVariable
from .offset_destination import OffsetDestination
from .property_variable import PropertyVariable
from .start_destination import StartDestination
from .string_value import StringValue
from .string_variable import StringVariable
from .url_value import UrlValue
from .url_variable import UrlVariable

ArrayValue.update_forward_refs()
ArrayVariable.update_forward_refs()
BooleanValue.update_forward_refs()
BooleanVariable.update_forward_refs()
ColorValue.update_forward_refs()
ColorVariable.update_forward_refs()
ContentText.update_forward_refs()
ContentUrl.update_forward_refs()
DictValue.update_forward_refs()
DictVariable.update_forward_refs()
DivAbsoluteEdgeInsets.update_forward_refs()
DivAccessibility.update_forward_refs()
DivAction.update_forward_refs()
DivActionMenuItem.update_forward_refs()
DivActionAnimatorStart.update_forward_refs()
DivActionAnimatorStop.update_forward_refs()
DivActionArrayInsertValue.update_forward_refs()
DivActionArrayRemoveValue.update_forward_refs()
DivActionArraySetValue.update_forward_refs()
DivActionClearFocus.update_forward_refs()
DivActionCopyToClipboard.update_forward_refs()
DivActionCustom.update_forward_refs()
DivActionDictSetValue.update_forward_refs()
DivActionDownload.update_forward_refs()
DivActionFocusElement.update_forward_refs()
DivActionHideTooltip.update_forward_refs()
DivActionScrollBy.update_forward_refs()
DivActionScrollTo.update_forward_refs()
DivActionSetState.update_forward_refs()
DivActionSetStoredValue.update_forward_refs()
DivActionSetVariable.update_forward_refs()
DivActionShowTooltip.update_forward_refs()
DivActionSubmit.update_forward_refs()
DivActionSubmitRequest.update_forward_refs()
RequestHeader.update_forward_refs()
DivActionTimer.update_forward_refs()
DivActionUpdateStructure.update_forward_refs()
DivActionVideo.update_forward_refs()
DivAnimation.update_forward_refs()
DivAnimatorBase.update_forward_refs()
DivAppearanceSetTransition.update_forward_refs()
DivAspect.update_forward_refs()
DivBase.update_forward_refs()
DivBlur.update_forward_refs()
DivBorder.update_forward_refs()
DivChangeBoundsTransition.update_forward_refs()
DivChangeSetTransition.update_forward_refs()
DivCircleShape.update_forward_refs()
DivCloudBackground.update_forward_refs()
DivCollectionItemBuilder.update_forward_refs()
DivCollectionItemBuilderPrototype.update_forward_refs()
DivColorAnimator.update_forward_refs()
DivContainer.update_forward_refs()
DivContainerSeparator.update_forward_refs()
DivCornersRadius.update_forward_refs()
DivCurrencyInputMask.update_forward_refs()
DivCustom.update_forward_refs()
DivData.update_forward_refs()
DivDataState.update_forward_refs()
DivDefaultIndicatorItemPlacement.update_forward_refs()
DivDimension.update_forward_refs()
DivDisappearAction.update_forward_refs()
DivDownloadCallbacks.update_forward_refs()
DivEdgeInsets.update_forward_refs()
DivExtension.update_forward_refs()
DivFadeTransition.update_forward_refs()
DivFilterRtlMirror.update_forward_refs()
DivFixedCount.update_forward_refs()
DivFixedLengthInputMask.update_forward_refs()
DivFixedLengthInputMaskPatternElement.update_forward_refs()
DivFixedSize.update_forward_refs()
DivFixedTranslation.update_forward_refs()
DivFocus.update_forward_refs()
DivFocusNextFocusIds.update_forward_refs()
DivFunction.update_forward_refs()
DivFunctionArgument.update_forward_refs()
DivGallery.update_forward_refs()
DivGifImage.update_forward_refs()
DivGrid.update_forward_refs()
DivImage.update_forward_refs()
DivImageBackground.update_forward_refs()
DivIndicator.update_forward_refs()
DivInfinityCount.update_forward_refs()
DivInput.update_forward_refs()
DivInputNativeInterface.update_forward_refs()
DivInputFilterExpression.update_forward_refs()
DivInputFilterRegex.update_forward_refs()
DivInputMaskBase.update_forward_refs()
DivInputValidatorBase.update_forward_refs()
DivInputValidatorExpression.update_forward_refs()
DivInputValidatorRegex.update_forward_refs()
DivLayoutProvider.update_forward_refs()
DivLinearGradient.update_forward_refs()
DivLinearGradientColorPoint.update_forward_refs()
DivMatchParentSize.update_forward_refs()
DivNeighbourPageSize.update_forward_refs()
DivNinePatchBackground.update_forward_refs()
DivNumberAnimator.update_forward_refs()
DivPageContentSize.update_forward_refs()
DivPageSize.update_forward_refs()
DivPageTransformationOverlap.update_forward_refs()
DivPageTransformationSlide.update_forward_refs()
DivPager.update_forward_refs()
DivPatch.update_forward_refs()
DivPatchChange.update_forward_refs()
DivPercentageSize.update_forward_refs()
DivPercentageTranslation.update_forward_refs()
DivPhoneInputMask.update_forward_refs()
DivPivotFixed.update_forward_refs()
DivPivotPercentage.update_forward_refs()
DivPoint.update_forward_refs()
DivRadialGradient.update_forward_refs()
DivRadialGradientColorPoint.update_forward_refs()
DivRadialGradientFixedCenter.update_forward_refs()
DivRadialGradientRelativeCenter.update_forward_refs()
DivRadialGradientRelativeRadius.update_forward_refs()
DivRotationTransformation.update_forward_refs()
DivRoundedRectangleShape.update_forward_refs()
DivScaleTransition.update_forward_refs()
DivSelect.update_forward_refs()
DivSelectOption.update_forward_refs()
DivSeparator.update_forward_refs()
DivSeparatorDelimiterStyle.update_forward_refs()
DivShadow.update_forward_refs()
DivShapeDrawable.update_forward_refs()
DivSightAction.update_forward_refs()
DivSizeUnitValue.update_forward_refs()
DivSlideTransition.update_forward_refs()
DivSlider.update_forward_refs()
DivSliderRange.update_forward_refs()
DivSliderTextStyle.update_forward_refs()
DivSolidBackground.update_forward_refs()
DivState.update_forward_refs()
DivStateState.update_forward_refs()
DivStretchIndicatorItemPlacement.update_forward_refs()
DivStroke.update_forward_refs()
DivStrokeStyleDashed.update_forward_refs()
DivStrokeStyleSolid.update_forward_refs()
DivSwitch.update_forward_refs()
DivTabs.update_forward_refs()
DivTabsItem.update_forward_refs()
DivTabsTabTitleDelimiter.update_forward_refs()
DivTabsTabTitleStyle.update_forward_refs()
DivText.update_forward_refs()
DivTextEllipsis.update_forward_refs()
DivTextImage.update_forward_refs()
ImageAccessibility.update_forward_refs()
DivTextRange.update_forward_refs()
DivTextRangeBorder.update_forward_refs()
DivTextRangeMaskBase.update_forward_refs()
DivTextRangeMaskParticles.update_forward_refs()
DivTextRangeMaskSolid.update_forward_refs()
DivTimer.update_forward_refs()
DivTooltip.update_forward_refs()
DivTooltipModeModal.update_forward_refs()
DivTooltipModeNonModal.update_forward_refs()
DivTransform.update_forward_refs()
DivTransitionBase.update_forward_refs()
DivTranslationTransformation.update_forward_refs()
DivTrigger.update_forward_refs()
DivVideo.update_forward_refs()
DivVideoSource.update_forward_refs()
DivVideoSourceResolution.update_forward_refs()
DivVisibilityAction.update_forward_refs()
DivWrapContentSize.update_forward_refs()
EndDestination.update_forward_refs()
IndexDestination.update_forward_refs()
IntegerValue.update_forward_refs()
IntegerVariable.update_forward_refs()
NumberValue.update_forward_refs()
NumberVariable.update_forward_refs()
OffsetDestination.update_forward_refs()
PropertyVariable.update_forward_refs()
StartDestination.update_forward_refs()
StringValue.update_forward_refs()
StringVariable.update_forward_refs()
UrlValue.update_forward_refs()
UrlVariable.update_forward_refs()

__all__ = (
    "AccessibilityType",
    "ArrayValue",
    "ArrayVariable",
    "BooleanValue",
    "BooleanVariable",
    "ColorValue",
    "ColorVariable",
    "ContentText",
    "ContentUrl",
    "DelimiterStyleOrientation",
    "DictValue",
    "DictVariable",
    "Div",
    "DivAbsoluteEdgeInsets",
    "DivAccessibility",
    "DivAccessibilityMode",
    "DivAccessibilityType",
    "DivAction",
    "DivActionAnimatorStart",
    "DivActionAnimatorStop",
    "DivActionArrayInsertValue",
    "DivActionArrayRemoveValue",
    "DivActionArraySetValue",
    "DivActionClearFocus",
    "DivActionCopyToClipboard",
    "DivActionCopyToClipboardContent",
    "DivActionCustom",
    "DivActionDictSetValue",
    "DivActionDownload",
    "DivActionFocusElement",
    "DivActionHideTooltip",
    "DivActionMenuItem",
    "DivActionScrollBy",
    "DivActionScrollByOverflow",
    "DivActionScrollDestination",
    "DivActionScrollTo",
    "DivActionSetState",
    "DivActionSetStoredValue",
    "DivActionSetVariable",
    "DivActionShowTooltip",
    "DivActionSubmit",
    "DivActionSubmitRequest",
    "DivActionTarget",
    "DivActionTimer",
    "DivActionTimerAction",
    "DivActionTyped",
    "DivActionUpdateStructure",
    "DivActionVideo",
    "DivActionVideoAction",
    "DivAlignmentHorizontal",
    "DivAlignmentVertical",
    "DivAnimation",
    "DivAnimationDirection",
    "DivAnimationInterpolator",
    "DivAnimationName",
    "DivAnimator",
    "DivAnimatorBase",
    "DivAppearanceSetTransition",
    "DivAppearanceTransition",
    "DivAspect",
    "DivBackground",
    "DivBase",
    "DivBlendMode",
    "DivBlur",
    "DivBorder",
    "DivChangeBoundsTransition",
    "DivChangeSetTransition",
    "DivChangeTransition",
    "DivCircleShape",
    "DivCloudBackground",
    "DivCollectionItemBuilder",
    "DivCollectionItemBuilderPrototype",
    "DivColorAnimator",
    "DivContainer",
    "DivContainerLayoutMode",
    "DivContainerOrientation",
    "DivContainerSeparator",
    "DivContentAlignmentHorizontal",
    "DivContentAlignmentVertical",
    "DivCornersRadius",
    "DivCount",
    "DivCurrencyInputMask",
    "DivCustom",
    "DivData",
    "DivDataState",
    "DivDefaultIndicatorItemPlacement",
    "DivDimension",
    "DivDisappearAction",
    "DivDownloadCallbacks",
    "DivDrawable",
    "DivEdgeInsets",
    "DivEvaluableType",
    "DivExtension",
    "DivFadeTransition",
    "DivFilter",
    "DivFilterRtlMirror",
    "DivFixedCount",
    "DivFixedLengthInputMask",
    "DivFixedLengthInputMaskPatternElement",
    "DivFixedSize",
    "DivFixedTranslation",
    "DivFocus",
    "DivFocusNextFocusIds",
    "DivFontWeight",
    "DivFunction",
    "DivFunctionArgument",
    "DivGallery",
    "DivGalleryCrossContentAlignment",
    "DivGalleryOrientation",
    "DivGalleryScrollMode",
    "DivGalleryScrollbar",
    "DivGifImage",
    "DivGrid",
    "DivImage",
    "DivImageBackground",
    "DivImageScale",
    "DivIndicator",
    "DivIndicatorAnimation",
    "DivIndicatorItemPlacement",
    "DivInfinityCount",
    "DivInput",
    "DivInputAutocapitalization",
    "DivInputEnterKeyType",
    "DivInputFilter",
    "DivInputFilterExpression",
    "DivInputFilterRegex",
    "DivInputKeyboardType",
    "DivInputMask",
    "DivInputMaskBase",
    "DivInputNativeInterface",
    "DivInputValidator",
    "DivInputValidatorBase",
    "DivInputValidatorExpression",
    "DivInputValidatorRegex",
    "DivLayoutProvider",
    "DivLineStyle",
    "DivLinearGradient",
    "DivLinearGradientColorPoint",
    "DivMatchParentSize",
    "DivNeighbourPageSize",
    "DivNinePatchBackground",
    "DivNumberAnimator",
    "DivPageContentSize",
    "DivPageSize",
    "DivPageTransformation",
    "DivPageTransformationOverlap",
    "DivPageTransformationSlide",
    "DivPager",
    "DivPagerItemAlignment",
    "DivPagerLayoutMode",
    "DivPagerOrientation",
    "DivPatch",
    "DivPatchChange",
    "DivPatchMode",
    "DivPercentageSize",
    "DivPercentageTranslation",
    "DivPhoneInputMask",
    "DivPivot",
    "DivPivotFixed",
    "DivPivotPercentage",
    "DivPoint",
    "DivRadialGradient",
    "DivRadialGradientCenter",
    "DivRadialGradientColorPoint",
    "DivRadialGradientFixedCenter",
    "DivRadialGradientRadius",
    "DivRadialGradientRelativeCenter",
    "DivRadialGradientRelativeRadius",
    "DivRadialGradientRelativeRadiusValue",
    "DivRotationTransformation",
    "DivRoundedRectangleShape",
    "DivScaleTransition",
    "DivSelect",
    "DivSelectOption",
    "DivSeparator",
    "DivSeparatorDelimiterStyle",
    "DivShadow",
    "DivShape",
    "DivShapeDrawable",
    "DivSightAction",
    "DivSize",
    "DivSizeUnit",
    "DivSizeUnitValue",
    "DivSlideTransition",
    "DivSlideTransitionEdge",
    "DivSlider",
    "DivSliderRange",
    "DivSliderTextStyle",
    "DivSolidBackground",
    "DivState",
    "DivStateState",
    "DivStretchIndicatorItemPlacement",
    "DivStroke",
    "DivStrokeStyle",
    "DivStrokeStyleDashed",
    "DivStrokeStyleSolid",
    "DivSwitch",
    "DivTabs",
    "DivTabsItem",
    "DivTabsTabTitleDelimiter",
    "DivTabsTabTitleStyle",
    "DivText",
    "DivTextAlignmentVertical",
    "DivTextEllipsis",
    "DivTextGradient",
    "DivTextImage",
    "DivTextRange",
    "DivTextRangeBackground",
    "DivTextRangeBorder",
    "DivTextRangeMask",
    "DivTextRangeMaskBase",
    "DivTextRangeMaskParticles",
    "DivTextRangeMaskSolid",
    "DivTextTruncate",
    "DivTimer",
    "DivTooltip",
    "DivTooltipMode",
    "DivTooltipModeModal",
    "DivTooltipModeNonModal",
    "DivTooltipPosition",
    "DivTransform",
    "DivTransformation",
    "DivTransitionBase",
    "DivTransitionSelector",
    "DivTransitionTrigger",
    "DivTranslation",
    "DivTranslationTransformation",
    "DivTrigger",
    "DivTriggerMode",
    "DivTypedValue",
    "DivVariable",
    "DivVideo",
    "DivVideoScale",
    "DivVideoSource",
    "DivVideoSourceResolution",
    "DivVisibility",
    "DivVisibilityAction",
    "DivWrapContentSize",
    "EndDestination",
    "ImageAccessibility",
    "ImageIndexingDirection",
    "IndexDestination",
    "IntegerValue",
    "IntegerVariable",
    "NumberValue",
    "NumberVariable",
    "OffsetDestination",
    "PropertyVariable",
    "RequestHeader",
    "RequestMethod",
    "StartDestination",
    "StringValue",
    "StringVariable",
    "TabTitleStyleAnimationType",
    "UrlValue",
    "UrlVariable",
)



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field


# Gaussian image blur.
class DivBlur(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "blur",
        radius: typing.Optional[typing.Union[Expr, int]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            radius=radius,
            **kwargs,
        )

    type: str = Field(default="blur")
    radius: typing.Union[Expr, int] = Field(
        description=(
            "Blur radius. Defines how many pixels blend into each other. "
            "Specified in: `dp`."
        )
    )


DivBlur.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field


class DivAnimationDirection(str, enum.Enum):
    NORMAL = "normal"
    REVERSE = "reverse"
    ALTERNATE = "alternate"
    ALTERNATE_REVERSE = "alternate_reverse"



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Expr, Field

from . import div_fixed_size, div_match_parent_size, div_wrap_content_size

DivSize = Union[
    div_fixed_size.DivFixedSize,
    div_match_parent_size.DivMatchParentSize,
    div_wrap_content_size.DivWrapContentSize,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_fixed_size, div_stroke


# Circle.
class DivCircleShape(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "circle",
        background_color: typing.Optional[typing.Union[Expr, str]] = None,
        radius: typing.Optional[div_fixed_size.DivFixedSize] = None,
        stroke: typing.Optional[div_stroke.DivStroke] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            background_color=background_color,
            radius=radius,
            stroke=stroke,
            **kwargs,
        )

    type: str = Field(default="circle")
    background_color: typing.Optional[typing.Union[Expr, str]] = Field(
        format="color", description="Fill color."
    )
    radius: typing.Optional[div_fixed_size.DivFixedSize] = Field(description="Radius.")
    stroke: typing.Optional[div_stroke.DivStroke] = Field(description="Stroke style.")


DivCircleShape.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_fixed_size


# Element size adjusts to a parent element.
class DivStretchIndicatorItemPlacement(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "stretch",
        item_spacing: typing.Optional[div_fixed_size.DivFixedSize] = None,
        max_visible_items: typing.Optional[typing.Union[Expr, int]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            item_spacing=item_spacing,
            max_visible_items=max_visible_items,
            **kwargs,
        )

    type: str = Field(default="stretch")
    item_spacing: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Spacing between indicator centers."
    )
    max_visible_items: typing.Optional[typing.Union[Expr, int]] = Field(
        description="Maximum number of visible indicators."
    )


DivStretchIndicatorItemPlacement.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_change_transition


# Animations.
class DivChangeSetTransition(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "set",
        items: typing.Optional[
            typing.Sequence[div_change_transition.DivChangeTransition]
        ] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            items=items,
            **kwargs,
        )

    type: str = Field(default="set")
    items: typing.Sequence[div_change_transition.DivChangeTransition] = Field(
        min_items=1, description="List of animations."
    )


DivChangeSetTransition.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_size_unit


# Element dimension value.
class DivDimension(BaseDiv):
    def __init__(
        self,
        *,
        unit: typing.Optional[typing.Union[Expr, div_size_unit.DivSizeUnit]] = None,
        value: typing.Optional[typing.Union[Expr, float]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            unit=unit,
            value=value,
            **kwargs,
        )

    unit: typing.Optional[typing.Union[Expr, div_size_unit.DivSizeUnit]] = Field()
    value: typing.Union[Expr, float] = Field(description="Value.")


DivDimension.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import (
    div_alignment_horizontal,
    div_alignment_vertical,
    div_filter,
    div_image_scale,
)


# Background image.
class DivImageBackground(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "image",
        alpha: typing.Optional[typing.Union[Expr, float]] = None,
        content_alignment_horizontal: typing.Optional[
            typing.Union[Expr, div_alignment_horizontal.DivAlignmentHorizontal]
        ] = None,
        content_alignment_vertical: typing.Optional[
            typing.Union[Expr, div_alignment_vertical.DivAlignmentVertical]
        ] = None,
        filters: typing.Optional[typing.Sequence[div_filter.DivFilter]] = None,
        image_url: typing.Optional[typing.Union[Expr, str]] = None,
        preload_required: typing.Optional[typing.Union[Expr, bool]] = None,
        scale: typing.Optional[typing.Union[Expr, div_image_scale.DivImageScale]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            alpha=alpha,
            content_alignment_horizontal=content_alignment_horizontal,
            content_alignment_vertical=content_alignment_vertical,
            filters=filters,
            image_url=image_url,
            preload_required=preload_required,
            scale=scale,
            **kwargs,
        )

    type: str = Field(default="image")
    alpha: typing.Optional[typing.Union[Expr, float]] = Field(
        description="Image transparency."
    )
    content_alignment_horizontal: typing.Optional[
        typing.Union[Expr, div_alignment_horizontal.DivAlignmentHorizontal]
    ] = Field(description="Horizontal image alignment.")
    content_alignment_vertical: typing.Optional[
        typing.Union[Expr, div_alignment_vertical.DivAlignmentVertical]
    ] = Field(description="Vertical image alignment.")
    filters: typing.Optional[typing.Sequence[div_filter.DivFilter]] = Field(
        description="Image filters."
    )
    image_url: typing.Union[Expr, str] = Field(format="uri", description="Image URL.")
    preload_required: typing.Optional[typing.Union[Expr, bool]] = Field(
        description="Background image must be loaded before the display."
    )
    scale: typing.Optional[typing.Union[Expr, div_image_scale.DivImageScale]] = Field(
        description="Image scaling."
    )


DivImageBackground.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_absolute_edge_insets


# Image in 9-patch format (https://developer.android.com/studio/write/draw9patch).
class DivNinePatchBackground(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "nine_patch_image",
        image_url: typing.Optional[typing.Union[Expr, str]] = None,
        insets: typing.Optional[div_absolute_edge_insets.DivAbsoluteEdgeInsets] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            image_url=image_url,
            insets=insets,
            **kwargs,
        )

    type: str = Field(default="nine_patch_image")
    image_url: typing.Union[Expr, str] = Field(format="uri", description="Image URL.")
    insets: div_absolute_edge_insets.DivAbsoluteEdgeInsets = Field(
        description=(
            "Margins that break the image into parts using the same "
            "rules as the CSS`border-image-slice` "
            "property(https://developer.mozilla.org/en-US/docs/Web/CSS/b"
            "order-image-slice)."
        )
    )


DivNinePatchBackground.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import (
    div,
    div_function,
    div_timer,
    div_transition_selector,
    div_trigger,
    div_variable,
)


# Root structure.
class DivData(BaseDiv):
    def __init__(
        self,
        *,
        functions: typing.Optional[typing.Sequence[div_function.DivFunction]] = None,
        log_id: typing.Optional[typing.Union[Expr, str]] = None,
        states: typing.Optional[typing.Sequence[DivDataState]] = None,
        timers: typing.Optional[typing.Sequence[div_timer.DivTimer]] = None,
        transition_animation_selector: typing.Optional[
            typing.Union[Expr, div_transition_selector.DivTransitionSelector]
        ] = None,
        variable_triggers: typing.Optional[
            typing.Sequence[div_trigger.DivTrigger]
        ] = None,
        variables: typing.Optional[typing.Sequence[div_variable.DivVariable]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            functions=functions,
            log_id=log_id,
            states=states,
            timers=timers,
            transition_animation_selector=transition_animation_selector,
            variable_triggers=variable_triggers,
            variables=variables,
            **kwargs,
        )

    functions: typing.Optional[typing.Sequence[div_function.DivFunction]] = Field(
        description="User functions."
    )
    log_id: typing.Union[Expr, str] = Field(description="Logging ID.")
    states: typing.Sequence[DivDataState] = Field(
        min_items=1,
        description=(
            "A set of visual element states. Each element can have a few "
            "states with adifferent layout. The states are displayed "
            "strictly one by one and switched "
            "using[action](div-action.md)."
        ),
    )
    timers: typing.Optional[typing.Sequence[div_timer.DivTimer]] = Field(
        description="List of timers."
    )
    transition_animation_selector: typing.Optional[
        typing.Union[Expr, div_transition_selector.DivTransitionSelector]
    ] = Field(description="Events that trigger transition animations. @deprecated")
    variable_triggers: typing.Optional[typing.Sequence[div_trigger.DivTrigger]] = Field(
        description="Triggers for changing variables."
    )
    variables: typing.Optional[typing.Sequence[div_variable.DivVariable]] = Field(
        description="Declaration of variables that can be used in an element."
    )


class DivDataState(BaseDiv):
    def __init__(
        self,
        *,
        div: typing.Optional[div.Div] = None,
        state_id: typing.Optional[typing.Union[Expr, int]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            div=div,
            state_id=state_id,
            **kwargs,
        )

    div: div.Div = Field(description="Contents.")
    state_id: typing.Union[Expr, int] = Field(description="State ID.")


DivDataState.update_forward_refs()


DivData.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_typed_value


# Sets the value in the dictionary by the specified key. Deletes the key if the
# value is not set.
class DivActionDictSetValue(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "dict_set_value",
        key: typing.Optional[typing.Union[Expr, str]] = None,
        value: typing.Optional[div_typed_value.DivTypedValue] = None,
        variable_name: typing.Optional[typing.Union[Expr, str]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            key=key,
            value=value,
            variable_name=variable_name,
            **kwargs,
        )

    type: str = Field(default="dict_set_value")
    key: typing.Union[Expr, str] = Field()
    value: typing.Optional[div_typed_value.DivTypedValue] = Field()
    variable_name: typing.Union[Expr, str] = Field()


DivActionDictSetValue.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Expr, Field

from . import div_tooltip_mode_modal, div_tooltip_mode_non_modal

DivTooltipMode = Union[
    div_tooltip_mode_non_modal.DivTooltipModeNonModal,
    div_tooltip_mode_modal.DivTooltipModeModal,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_size_unit_value


# Element size adjusts to a parent element.
class DivMatchParentSize(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "match_parent",
        max_size: typing.Optional[div_size_unit_value.DivSizeUnitValue] = None,
        min_size: typing.Optional[div_size_unit_value.DivSizeUnitValue] = None,
        weight: typing.Optional[typing.Union[Expr, float]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            max_size=max_size,
            min_size=min_size,
            weight=weight,
            **kwargs,
        )

    type: str = Field(default="match_parent")
    max_size: typing.Optional[div_size_unit_value.DivSizeUnitValue] = Field(
        description="Maximum size of an element."
    )
    min_size: typing.Optional[div_size_unit_value.DivSizeUnitValue] = Field(
        description="Minimum size of an element."
    )
    weight: typing.Optional[typing.Union[Expr, float]] = Field(
        description=(
            "Weight when distributing free space between elements with "
            "the size type`match_parent` inside an element. If the "
            "weight isn't specified, the elementswill divide the place "
            "equally."
        )
    )


DivMatchParentSize.update_forward_refs()



import pytest

from pydivkit.core import Expr


@pytest.mark.parametrize("input,res", (("@{string}", Expr("@{string}")),))
def test_expr(input, res):
    assert str(Expr(input)) == str(res)


@pytest.mark.parametrize("input", ("@{before", "{before", "after}", "notcontains"))
def test_expr_fail(input):
    with pytest.raises(ValueError):
        Expr(input)



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Expr, Field

from . import (
    array_variable,
    boolean_variable,
    color_variable,
    dict_variable,
    integer_variable,
    number_variable,
    property_variable,
    string_variable,
    url_variable,
)

DivVariable = Union[
    string_variable.StringVariable,
    number_variable.NumberVariable,
    integer_variable.IntegerVariable,
    boolean_variable.BooleanVariable,
    color_variable.ColorVariable,
    url_variable.UrlVariable,
    dict_variable.DictVariable,
    array_variable.ArrayVariable,
    property_variable.PropertyVariable,
]



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing
from typing import Union

from pydivkit.core import BaseDiv, Expr, Field

from . import div_default_indicator_item_placement, div_stretch_indicator_item_placement

DivIndicatorItemPlacement = Union[
    div_default_indicator_item_placement.DivDefaultIndicatorItemPlacement,
    div_stretch_indicator_item_placement.DivStretchIndicatorItemPlacement,
]



import enum
from typing import Any, Mapping, Optional, Sequence, Union

import pytest

from pydivkit.core import BaseDiv, BaseEntity, Expr, Field, Ref


# ---------------------------------------------------------------------------
# BaseEntity: construction and field assignment
# ---------------------------------------------------------------------------
class TestBaseEntityInit:
    def test_create_simple_entity(self):
        class SimpleEntity(BaseEntity):
            name: str = Field()

        entity = SimpleEntity(name="hello")
        assert entity.name == "hello"

    def test_field_with_default(self):
        class DefaultEntity(BaseEntity):
            color: str = Field(default="red")

        entity = DefaultEntity()
        assert entity.color == "red"

    def test_optional_field_default_none(self):
        """Optional field defaults to None when not provided."""

        class OptEntity(BaseEntity):
            label: Optional[str] = Field()

        entity = OptEntity()
        assert entity.label is None

    def test_sequence_field(self):
        class SeqEntity(BaseEntity):
            items: Sequence[int] = Field()

        entity = SeqEntity(items=[1, 2, 3])
        assert entity.items == [1, 2, 3]

    def test_sequence_field_wrong_type_raises(self):
        class SeqEntity2(BaseEntity):
            items: Sequence[int] = Field()

        with pytest.raises(ValueError, match="has wrong type"):
            SeqEntity2(items=42)

    def test_mapping_field(self):
        class MapEntity(BaseEntity):
            data: Mapping[str, int] = Field()

        entity = MapEntity(data={"a": 1, "b": 2})
        assert entity.data == {"a": 1, "b": 2}

    def test_mapping_field_wrong_type_raises(self):
        class MapEntity2(BaseEntity):
            data: Mapping[str, int] = Field()

        with pytest.raises(ValueError, match="has wrong type"):
            MapEntity2(data=42)

    def test_any_field_passthrough(self):
        class AnyEntity(BaseEntity):
            data: Any = Field()

        value = {"key": [1, 2, 3]}
        entity = AnyEntity(data=value)
        assert entity.data is value

    def test_union_field_exact_match(self):
        class UnionEntity(BaseEntity):
            val: Union[int, str] = Field()

        entity = UnionEntity(val=42)
        assert entity.val == 42

    def test_union_field_no_match_raises(self):
        class UnionEntity2(BaseEntity):
            val: Union[int, float] = Field()

        with pytest.raises(ValueError, match="has wrong type"):
            UnionEntity2(val=[])

    def test_type_coercion(self):
        """Values are coerced to field type (e.g. int -> float)."""

        class CoerceEntity(BaseEntity):
            val: float = Field()

        entity = CoerceEntity(val=42)
        assert isinstance(entity.val, float)
        assert entity.val == 42.0

    def test_unknown_attribute_raises(self):
        class StrictEntity(BaseEntity):
            val: int = Field(default=0)

        entity = StrictEntity()
        with pytest.raises(KeyError, match="has no attribute 'unknown'"):
            entity.unknown = 42

    def test_wrong_type_raises(self):
        """Passing an incompatible value type raises ValueError."""

        class StrictTypeEntity(BaseEntity):
            val: int = Field()

        with pytest.raises(ValueError, match="has wrong type"):
            StrictTypeEntity(val=[])


# ---------------------------------------------------------------------------
# BaseEntity: type field validation
# ---------------------------------------------------------------------------
class TestBaseEntityTypeField:
    def test_type_field_wrong_value_raises(self):
        class TypedEntity(BaseEntity):
            type: str = Field(name="type", default="expected")

        with pytest.raises(ValueError, match="has wrong value"):
            TypedEntity(type="wrong")

    def test_type_field_correct_value(self):
        class TypedEntity2(BaseEntity):
            type: str = Field(name="type", default="correct")

        entity = TypedEntity2(type="correct")
        assert entity.type == "correct"


# ---------------------------------------------------------------------------
# BaseEntity: dict() serialization
# ---------------------------------------------------------------------------
class TestBaseEntityDict:
    def test_simple_dict(self):
        class DictEntity(BaseEntity):
            name: str = Field(default="default")
            count: Optional[int] = Field()

        entity = DictEntity(name="test", count=5)
        assert entity.dict() == {"name": "test", "count": 5}

    def test_dict_skips_none(self):
        class NoneEntity(BaseEntity):
            name: str = Field(default="x")
            opt: Optional[int] = Field()

        entity = NoneEntity(name="hello")
        assert entity.dict() == {"name": "hello"}

    def test_dict_with_nested_entity(self):
        class Inner(BaseEntity):
            x: int = Field(default=1)

        class Outer(BaseEntity):
            inner: Inner = Field()

        entity = Outer(inner=Inner(x=10))
        assert entity.dict() == {"inner": {"x": 10}}

    def test_dict_with_sequence(self):
        class SeqDictEntity(BaseEntity):
            items: Sequence[int] = Field()

        entity = SeqDictEntity(items=[1, 2, 3])
        assert entity.dict() == {"items": [1, 2, 3]}

    def test_dict_with_mapping(self):
        class MapDictEntity(BaseEntity):
            data: Mapping[str, Any] = Field()

        entity = MapDictEntity(data={"key": "value"})
        assert entity.dict() == {"data": {"key": "value"}}

    def test_dict_with_expr(self):
        class ExprEntity(BaseEntity):
            val: Union[str, Expr] = Field()

        entity = ExprEntity(val=Expr("@{some_var}"))
        assert entity.dict() == {"val": "@{some_var}"}

    def test_dict_with_enum(self):
        class Color(enum.Enum):
            RED = "red"
            GREEN = "green"

        class EnumEntity(BaseEntity):
            color: Color = Field(default=Color.RED)

        entity = EnumEntity(color=Color.GREEN)
        assert entity.dict() == {"color": "green"}

    def test_dict_with_nested_entity_in_sequence(self):
        class Item(BaseEntity):
            x: int = Field(default=1)

        class ListEntity(BaseEntity):
            items: Sequence[Item] = Field()

        entity = ListEntity(items=[Item(x=10)])
        assert entity.dict() == {"items": [{"x": 10}]}


# ---------------------------------------------------------------------------
# BaseEntity: template_name
# ---------------------------------------------------------------------------
class TestBaseEntityTemplateName:
    def test_default_template_name(self):
        class NameEntity(BaseEntity):
            val: int = Field(default=0)

        expected = f"{NameEntity.__module__}.{NameEntity.__name__}"
        assert NameEntity.template_name == expected

    def test_custom_template_name(self):
        class CustomNameEntity(BaseEntity):
            __template_name__ = "custom.template"
            val: int = Field(default=0)

        assert CustomNameEntity.template_name == "custom.template"


# ---------------------------------------------------------------------------
# BaseEntity: related_templates
# ---------------------------------------------------------------------------
class TestBaseEntityRelatedTemplates:
    def test_empty_for_plain_entity(self):
        class PlainEntity(BaseEntity):
            val: int = Field(default=0)

        entity = PlainEntity()
        assert entity.related_templates() == set()


# ---------------------------------------------------------------------------
# BaseEntity: union with injected types (dict -> entity cast)
# ---------------------------------------------------------------------------
class TestBaseEntityUnionInjectedTypes:
    def test_union_field_dict_cast_to_entity(self):
        """Dict with 'type' field is cast to correct entity in a Union."""

        class TypeAlpha(BaseEntity):
            type: str = Field(name="type", default="alpha")
            val: int = Field(default=0)

        class TypeBeta(BaseEntity):
            type: str = Field(name="type", default="beta")
            name: str = Field(default="x")

        class Container(BaseEntity):
            child: Union[TypeAlpha, TypeBeta] = Field()

        entity = Container(child={"type": "alpha", "val": 42})
        assert isinstance(entity.child, TypeAlpha)
        assert entity.child.val == 42

    def test_union_field_dict_without_type_raises(self):
        class TypeGamma(BaseEntity):
            type: str = Field(name="type", default="gamma")
            val: int = Field(default=0)

        class TypeDelta(BaseEntity):
            type: str = Field(name="type", default="delta")
            val: int = Field(default=0)

        class Container2(BaseEntity):
            child: Union[TypeGamma, TypeDelta] = Field()

        with pytest.raises(ValueError, match="does not have field type"):
            Container2(child={"val": 42})

    def test_union_field_dict_unknown_type_raises(self):
        class TypeEpsilon(BaseEntity):
            type: str = Field(name="type", default="epsilon")
            val: int = Field(default=0)

        class TypeZeta(BaseEntity):
            type: str = Field(name="type", default="zeta")
            val: int = Field(default=0)

        class Container3(BaseEntity):
            child: Union[TypeEpsilon, TypeZeta] = Field()

        with pytest.raises(ValueError, match="does not contain type"):
            Container3(child={"type": "unknown", "val": 1})


# ---------------------------------------------------------------------------
# BaseDiv: template building
# ---------------------------------------------------------------------------
class TestBaseDivTemplate:
    def test_direct_subclass_is_not_template(self):
        class RawDiv(BaseDiv):
            name: str = Field(default="x")

        with pytest.raises(TypeError, match="is not a template"):
            RawDiv.template()

    def test_template_with_tpl_values(self):
        class CompBase(BaseDiv):
            type: str = Field(name="type", default="component")
            text: str = Field()

        class MyComponent(CompBase):
            text = "default_text"

        tpl = MyComponent.template()
        assert tpl["type"] == "component"
        assert tpl["text"] == "default_text"

    def test_template_with_ref_field(self):
        class RefBase(BaseDiv):
            type: str = Field(name="type", default="ref_comp")
            label: str = Field()

        class RefTemplate(RefBase):
            custom_label: str = Field()
            label = Ref(custom_label)

        tpl = RefTemplate.template()
        assert tpl["type"] == "ref_comp"
        assert "$label" in tpl
        assert tpl["$label"] == "custom_label"

    def test_template_caching(self):
        class CacheBase(BaseDiv):
            type: str = Field(name="type", default="cache_test")
            val: int = Field()

        class CachedComponent(CacheBase):
            val = 42

        tpl1 = CachedComponent.template()
        tpl2 = CachedComponent.template()
        assert tpl1 is tpl2

    def test_template_none_tpl_value_not_overridden(self):
        """Instantiating with None doesn't override template values."""

        class TplBase(BaseDiv):
            type: str = Field(name="type", default="tpl_base")
            color: str = Field()

        class TplChild(TplBase):
            color = "red"

        instance = TplChild()
        assert instance.dict().get("type") is not None

    def test_multiple_bases_raises(self):
        with pytest.raises(TypeError, match="cannot be uniquely identified"):

            class MultiBaseDiv(BaseDiv, BaseEntity):
                pass


# ---------------------------------------------------------------------------
# BaseDiv: schema()
# ---------------------------------------------------------------------------
class TestBaseDivSchema:
    def test_schema_returns_object_with_definitions(self):
        class SchBase(BaseDiv):
            type: str = Field(name="type", default="schema_test")
            val: int = Field()

        class SchChild(SchBase):
            pass

        schema = SchChild.schema()
        assert schema["type"] == "object"
        assert "definitions" in schema
        assert isinstance(schema["definitions"], dict)

    def test_schema_tpl_values_excluded(self):
        class SchBase2(BaseDiv):
            type: str = Field(name="type", default="sch_tpl")
            color: str = Field()
            size: int = Field()

        class SchChild2(SchBase2):
            color = "red"

        schema = SchChild2.schema()
        props = schema.get("properties", {})
        assert "color" not in props
        assert "size" in props

    def test_schema_with_exclude_fields(self):
        class SchBase3(BaseDiv):
            type: str = Field(name="type", default="sch_excl")
            a: str = Field()
            b: str = Field()

        class SchChild3(SchBase3):
            pass

        schema = SchChild3.schema(exclude_fields=["a"])
        props = schema.get("properties", {})
        assert "a" not in props

    def test_schema_with_nested_exclude(self):
        class SchBase4(BaseDiv):
            type: str = Field(name="type", default="sch_nest")
            a: str = Field()
            b: str = Field()

        class SchChild4(SchBase4):
            pass

        schema = SchChild4.schema(exclude_fields=["a.nested"])
        assert "definitions" in schema

    def test_schema_optional_not_required(self):
        class SchBase5(BaseDiv):
            type: str = Field(name="type", default="sch_opt")
            required_field: int = Field()
            optional_field: Optional[str] = Field()

        class SchChild5(SchBase5):
            pass

        schema = SchChild5.schema()
        required = schema.get("required", [])
        assert "required_field" in required
        assert "optional_field" not in required

    def test_schema_int_field(self):
        class IntBase(BaseDiv):
            type: str = Field(name="type", default="int_sch")
            count: int = Field()

        class IntChild(IntBase):
            pass

        schema = IntChild.schema()
        props = schema.get("properties", {})
        assert props["count"]["type"] == "integer"

    def test_schema_str_field(self):
        class StrBase(BaseDiv):
            type: str = Field(name="type", default="str_sch")
            name: str = Field()

        class StrChild(StrBase):
            pass

        schema = StrChild.schema()
        props = schema.get("properties", {})
        assert props["name"]["type"] == "string"

    def test_schema_float_field(self):
        class FloatBase(BaseDiv):
            type: str = Field(name="type", default="float_sch")
            rate: float = Field()

        class FloatChild(FloatBase):
            pass

        schema = FloatChild.schema()
        props = schema.get("properties", {})
        assert props["rate"]["type"] == "number"

    def test_schema_bool_field(self):
        class BoolBase(BaseDiv):
            type: str = Field(name="type", default="bool_sch")
            flag: bool = Field()

        class BoolChild(BoolBase):
            pass

        schema = BoolChild.schema()
        props = schema.get("properties", {})
        assert props["flag"]["type"] == "integer"
        assert props["flag"]["format"] == "boolean"

    def test_schema_sequence_field(self):
        class SeqBase(BaseDiv):
            type: str = Field(name="type", default="seq_sch")
            items: Sequence[int] = Field()

        class SeqChild(SeqBase):
            pass

        schema = SeqChild.schema()
        props = schema.get("properties", {})
        assert props["items"]["type"] == "array"
        assert props["items"]["items"]["type"] == "integer"

    def test_schema_mapping_field(self):
        class MapBase(BaseDiv):
            type: str = Field(name="type", default="map_sch")
            data: Mapping[str, Any] = Field()

        class MapChild(MapBase):
            pass

        schema = MapChild.schema()
        props = schema.get("properties", {})
        assert props["data"]["type"] == "object"
        assert props["data"]["additionalProperties"] is True

    def test_schema_union_field(self):
        class UnionBase(BaseDiv):
            type: str = Field(name="type", default="union_sch")
            val: Union[int, str] = Field()

        class UnionChild(UnionBase):
            pass

        schema = UnionChild.schema()
        props = schema.get("properties", {})
        assert "anyOf" in props["val"]
        type_values = [a["type"] for a in props["val"]["anyOf"]]
        assert "integer" in type_values
        assert "string" in type_values

    def test_schema_enum_field(self):
        class Direction(enum.Enum):
            LEFT = "left"
            RIGHT = "right"

        class EnumBase(BaseDiv):
            type: str = Field(name="type", default="enum_sch")
            dir: Direction = Field()

        class EnumChild(EnumBase):
            pass

        schema = EnumChild.schema()
        props = schema.get("properties", {})
        assert props["dir"]["type"] == "string"
        assert set(props["dir"]["enum"]) == {"left", "right"}

    def test_schema_nested_entity_field(self):
        class Nested(BaseEntity):
            x: int = Field(default=0)

        class NestBase(BaseDiv):
            type: str = Field(name="type", default="nest_sch")
            child: Nested = Field()

        class NestChild(NestBase):
            pass

        schema = NestChild.schema()
        props = schema.get("properties", {})
        assert "$ref" in props["child"]
        assert "Nested" in props["child"]["$ref"]
        assert "Nested" in schema["definitions"]

    def test_schema_field_with_description_and_constraints(self):
        class DescBase(BaseDiv):
            type: str = Field(name="type", default="desc_sch")
            count: int = Field(
                description="Item count",
                ge=0,
                le=100,
            )

        class DescChild(DescBase):
            pass

        schema = DescChild.schema()
        props = schema.get("properties", {})
        count_schema = props["count"]
        assert count_schema.get("description") == "Item count"
        assert count_schema.get("minimum") == 0
        assert count_schema.get("maximum") == 100

    def test_schema_type_field(self):
        """The 'type' field with a default generates enum schema."""

        class TypeBase(BaseDiv):
            type: str = Field(name="type", default="type_sch")
            val: int = Field()

        class TypeChild(TypeBase):
            pass

        schema = TypeChild.schema()
        props = schema.get("properties", {})
        type_prop = props["type"]
        assert type_prop["type"] == "string"
        assert "enum" in type_prop


# ---------------------------------------------------------------------------
# Integration with generated DivKit classes
# ---------------------------------------------------------------------------
class TestIntegration:
    def test_make_div_with_template(self):
        from pydivkit import DivContainer, DivText, make_div

        class MyTemplate(DivContainer):
            label: str = Field()
            items = [DivText(text=Ref(label))]

        result = make_div(MyTemplate(items=[], label="hello"))
        assert "templates" in result
        assert "card" in result
        tpl_name = MyTemplate.template_name
        assert tpl_name in result["templates"]

    def test_schema_with_generated_classes(self):
        from pydivkit import DivContainer, DivText

        class TextTemplate(DivContainer):
            label: str = Field()
            items = [DivText(text=Ref(label))]

        schema = TextTemplate.schema()
        assert schema["type"] == "object"
        assert "definitions" in schema



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field

from . import div_fixed_size


# Element size adjusts to a parent element.
class DivDefaultIndicatorItemPlacement(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "default",
        space_between_centers: typing.Optional[div_fixed_size.DivFixedSize] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            space_between_centers=space_between_centers,
            **kwargs,
        )

    type: str = Field(default="default")
    space_between_centers: typing.Optional[div_fixed_size.DivFixedSize] = Field(
        description="Spacing between indicator centers."
    )


DivDefaultIndicatorItemPlacement.update_forward_refs()



# Generated code. Do not modify.
# flake8: noqa: F401, F405, F811

from __future__ import annotations

import enum
import typing

from pydivkit.core import BaseDiv, Expr, Field


# Applies a new appearance to the content in `div-state'.
class DivActionSetState(BaseDiv):
    def __init__(
        self,
        *,
        type: str = "set_state",
        state_id: typing.Optional[typing.Union[Expr, str]] = None,
        temporary: typing.Optional[typing.Union[Expr, bool]] = None,
        **kwargs: typing.Any,
    ):
        super().__init__(
            type=type,
            state_id=state_id,
            temporary=temporary,
            **kwargs,
        )

    type: str = Field(default="set_state")
    state_id: typing.Union[Expr, str] = Field(
        description=(
            "The path of the state inside `state` that needs to be "
            "activated. Set in theformat "
            "`div_data_state_id/id/state_id'. Can be "
            "hierarchical:`div_data_state_id/id_1/state_id_1/../id_n/sta"
            "te_id_n`. Consistsof:`div_data_state_id` — the numeric "
            "value of the `state_id` of the `state`object in `data`'id` "
            "— the `id` value of the `state` object`state_id` — "
            "the`state_id` value of the `state` object in `state`"
        )
    )
    temporary: typing.Optional[typing.Union[Expr, bool]] = Field(
        description=(
            "Indicates a state change:`true` — the change is temporary "
            "and will switch to theoriginal one (default value) when the "
            "element is recreated`false` — the change ispermanent"
        )
    )


DivActionSetState.update_forward_refs()



from .entities import BaseDiv, BaseEntity
from .fields import Expr, Field, Ref
from .schema import (
    ExcludeFieldsType,
    SchemaType,
    _field_to_schema,
)
from .serialization import (
    _cast_value_type,
    _make_exclude_fields,
    _merge_types,
    _unpack_optional_type,
    _update_related_templates,
    dump,
)

__all__ = (
    "BaseEntity",
    "BaseDiv",
    "Field",
    "Ref",
    "Expr",
    "dump",
    "_cast_value_type",
    "_make_exclude_fields",
    "_merge_types",
    "_unpack_optional_type",
    "_update_related_templates",
    "_field_to_schema",
    "ExcludeFieldsType",
    "SchemaType",
)
